"""
Classes to generate flash-chart object and data
based on the original version

Julien Fache 'Fantomas' <fantomas42@gmail.com>
"""

class GraphObject(object):
    """Object to make the flash insertion"""
	
    @classmethod
    def render(cls, width=200, height=200, data_url='', swf_url_root='/', name='graph'):
        """Return HTML code to display the open-flash-chart.swf"""
        swf_infos = {'width': str(width), 'height': str(height), 'name': name,
                     'swf_url_root': swf_url_root, 'data_url': data_url}

	swf_object = ('<object classid="clsid:d27cdb6e-ae6d-11cf-96b8-444553540000" '
		      'codebase="http://fpdownload.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=8,0,0,0" '
		      'width="%(width)s" height="%(height)s" id="ie_%(name)s" align="middle">'
		      '<param name="allowScriptAccess" value="sameDomain" />'
		      '<param name="movie" value="%(swf_url_root)sopen-flash-chart.swf?width=%(width)s&height=%(height)s&data=%(data_url)s" />'
		      '<param name="quality" value="high" /><param name="bgcolor" value="#FFFFFF" />'
		      '<embed src="%(swf_url_root)sopen-flash-chart.swf?width=%(width)s&height=%(height)s&data=%(data_url)s" quality="high" '
		      'bgcolor="#FFFFFF" width="%(width)s" height="%(height)s" id="%(name)s" align="middle" '
		      'allowScriptAccess="sameDomain" type="application/x-shockwave-flash" '
		      'pluginspage="http://www.macromedia.com/go/getflashplayer" />'
		      '</object>') % swf_infos
	return swf_object

class Graph(object):
    """Generate a graph text-file"""
    line_default = '&line=3,#87421F&\r\n'
    true_false = {True: 'true', False: 'false'}

    def __init__(self):
        """Init the graph"""
        self.lines = []
        self.data = []
        
        self.pie_data = ''
        self.pie_value = ''
        self.pie_labels = ''
        self.pie_links = ''
        self.pie_colours = ''

        self.x_max = 0
        self.x_min = 0
        self.x_offset = ''
        self.x_tick_size = -1
        self.x_axis_steps = 1
        self.x_axis_3d = 0
        self.x_axis_colour = ''
        self.x_grid_color = ''
        self.x_legend = ''
        self.x_legend_size = 20
        self.x_legend_colour = '#000000'
        self.x_labels = []
        self.x_label_style_size = -1
        self.x_label_style_colour = '#000000'
        self.x_label_style_orientation = 0
        self.x_label_style_step = 1

        self.y_max = 20
        self.y_min = 0
        self.y_steps = 5
        self.y_axis_colour = ''
        self.y_grid_colour = ''
        self.y_legend = ''
        self.y_legend_size = 20
        self.y_legend_colour = '#000000'
        self.y_label_style_size = -1
        self.y_label_style_colour = '#000000'

        self.y2_max = 0
        self.y2_min = 0
        self.y2_lines = []
        self.y_right_legend = ''
        self.y_right_legend_size = 20
        self.y_right_legend_colour = '#000000'
        self.y_right_axis_colour = ''

        self.title_text = ''
        self.title_style = '{font-size: 30px; color: #000000}'

        self.bg_colour = ''
        self.bg_image = ''
        self.bg_image_x = ''
        self.bg_image_y = ''
        self.inner_bg_colour = ''
        self.inner_bg_colour_2 = ''
        self.inner_bg_angle = ''

        self.y_format = ''
        self.num_decimals = 0
        self.is_fixed_num_decimals_forced = ''
        self.is_decimal_separator_comma = ''    
        self.is_thousand_separator_disabled = ''
        self.tool_tip = ''

    def line_index(self):
        """Return the suffix for line chart"""
        if self.lines:
            return '_%s' % (len(self.lines) + 1)
        return ''

    def set_data(self, a):
        """Set line of data"""
        if not len(self.data):
            self.data.append('&values=%s&\r\n' % ','.join([str(v) for v in a]) )
        else:
            self.data.append('&values_%s=%s&\r\n' % (len(self.data) + 1, ','.join([str(v) for v in a])))

    def clear_data(self):
        """Clear all the data"""
        self.data = []

    def pie_values(self, values, labels, links=''):
        """Set data for pie"""
        self.pie_value = '&values=%s&\r\n' % ','.join([str(value) for value in values])
        self.pie_labels = '&pie_labels=%s&\r\n' % ','.join(labels)
        self.pie_links  = '&links=%s&\r\n' % ','.join(links)
		
    def clear_pie_data( self ):
        """Clear the data for pie"""
        self.pie_values = ''
        self.pie_labels = ''
        self.pie_links = ''

    def pie_slice_colours(self, colours):
        """Set the colours for the pie"""
        self.pie_colours = '&colours=%s&\r\n' % ','.join(colours)

    #X axis    
    def set_x_max(self, max_):
        self.x_max = int(max_)

    def set_x_min(self, min_):
        self.x_min = int(min_)

    def set_x_offset(self, val):
        self.x_offset = true_false(bool(value))

    def set_x_tick_size(self, size):
        self.x_tick_size = int(size)

    def set_x_axis_steps(self, steps):
        self.x_axis_steps = int(steps)

    def set_x_axis_3d(self, size):
        self.x_axis_3d = int(size);

    def set_x_axis_colour( self, axis, grid=''):
        self.x_axis_colour = axis
        self.x_grid_colour = grid

    def set_x_legend(self, text, size=-1, colour=''):
        self.x_legend = text
        
        if size > -1:
            self.x_legend_size = size
                
        if colour:
            self.x_legend_colour = colour

    def set_x_labels(self, labels):
        self.x_labels = labels

    x_label_grid_colour = '#000000'#undocumented variable
    def set_x_label_style( self, size, colour='', orientation=0, step=-1, grid_colour=''):
        #XXX see to integrate grid_colour in render()
        self.x_label_style_size = size

        if colour:
            self.x_label_style_colour = colour

        if grid_colour:
            self.x_label_grid_colour = grid_colour

        if orientation > 0:
            self.x_label_style_orientation = orientation

        if step > 0:
            self.x_label_style_step = step

    #Y axis    
    def set_y_max(self, max_):
        self.y_max = int(max_)

    def set_y_min(self, min_):
        self.y_min = int(min_)

    def set_y_label_steps(self, value):
        self.y_steps = int(value)

    def set_y_axis_colour( self, axis, grid=''):
        self.y_axis_colour = axis
        self.y_grid_colour = grid

    def set_y_legend(self, text, size=-1, colour=''):
        self.y_legend = text
        if size > -1:
            self.y_legend_size = size

        if colour:
            self.y_legend_colour = colour

    def set_y_label_style(self, size, colour=''):
        self.y_label_style_size = size

        if colour:
            self.y_label_style_colour = colour
    
    #Y2 axis    
    def set_y_right_max(self, max_):
        self.y2_max =int(max_)

    def set_y_right_min(self, min_):
        self.y2_min = int(min_)
        
    def attach_to_y_right_axis(self, data_number):
        """Set data for the right Y axis"""
        self.y2_lines.append(data_number)

    def set_y_right_legend(self, text, size=-1, colour=''):
        self.y_right_legend = text
        if size > -1:
            self.y_right_legend_size = size

        if colour:
            self.y_right_legend_colour = colour

    def set_y_right_axis_colour(self, colour):
        self.y_right_axis_colour = colour

    #Title
    def title(self, title, style=''):
        """Set the title with his style"""
        self.title_text = title
        if style:
            self.title_style = style

    #Background
    def set_bg_colour(self, colour):
        self.bg_colour = colour

    def set_bg_image(self, url, x='center', y='center'):
        self.bg_image = url
        self.bg_image_x = x
        self.bg_image_y = y

    def set_inner_background(self, col, col2='', angle=-1):
        self.inner_bg_colour = col

        if col2:
            self.inner_bg_colour_2 = col2
        
        if angle != -1:
            self.inner_bg_angle = angle

    #Number formatting
    def set_y_format(self, value):
        self.y_format = value	

    def set_num_decimals(self, value):
        self.num_decimals = int(value)

    def set_is_fixed_num_decimals_forced(self, value):
        self.is_fixed_num_decimals_forced = true_false(bool(value))

    def set_is_decimal_separator_comma(self, value):
        self.is_decimal_separator_comma = true_false(bool(value))

    def set_is_thousand_separator_disabled(self, value):
        self.is_thousand_separator_disabled = true_false(bool(value))

    #Tool Tip
    def set_tool_tip(self, tip):
        self.tool_tip = tip

    #Chart
    def line(self, width, colour='#000000', text='', size=''):
        """Add a line """
        chart = '&line%s=%s,%s,%s,%s&\r\n' % (
            self.line_index(), width, colour, text, size)

        self.lines.append(chart)

    def line_dot(self, width, dot_size, colour='#000000', text='', size=''):
        """Add a line dot"""
        chart = '&line_dot%s=%s,%s,%s,%s,%s&\r\n' % (
            self.line_index(), width, colour, text, size, dot_size)
        self.lines.append(chart)

    def line_hollow(self, width, dot_size, colour='#000000', text='', size=''):
        """Add a line hollow"""
        chart = '&line_hollow%s=%s,%s,%s,%s,%s&\r\n' % (
            self.line_index(), width, colour, text, size, dot_size)
        self.lines.append(chart)

    def area_hollow(self, width, dot_size, alpha=75, colour='#ff0000',
                    text='', size='', fill_colour=''):
        """Add an area hollow"""
        if not fill_color:
            fill_colour = colour
        chart = '&area_hollow%s=%s,%s,%s,%s,%s,%s,%s&\r\n' % (
            self.line_index(), width, dot_size, alpha,
            colour, text, size, fill_colour)
        self.lines.append(chart)

    def bar(self, alpha, colour, text='', size=''):
        """Add a bar"""
        chart = '&bar%s=%s,%s,%s,%s&\r\n' % (
            self.line_index(), alpha, colour, text, size)
        self.lines.append(chart)

    def bar_outline(self, alpha, colour, colour_outline, text='', size=''):
        """Add an outline bar"""
        chart = '&filled_bar%s=%s,%s,%s,%s,%s&\r\n' % (
            self.line_index(), alpha, colour, colour_outline, text, size)
        self.lines.append(chart)

    def bar_3d(self, alpha, colour, text='', size=''):
        """Add a 3d bar"""
        chart = '&bar_3d%s=%s,%s,%s,%s&\r\n' % (
            self.line_index(), alpha, colour, text, size)
        self.lines.append(chart)

    def bar_fade(self, alpha, colour, text='', size=''):
        """Add a fade bar"""
        chart = '&bar_fade%s=%s,%s,%s,%s&\r\n' % (
            self.line_index(), alpha, colour, text, size)
        self.lines.append(chart)

    def bar_sketch(self, alpha, offset, colour, colour_outline, text='', size=''):
        """Add a sketch bar"""
        chart = '&bar_sketch%s=%s,%s,%s,%s,%s,%s&\r\n' % (
            self.line_index(), alpha, offset, colour, colour_outline, text, size)
        self.lines.append(chart)

    def bar_glass(self, alpha, offset, colour, colour_outline, text='', size=''):
        """Add a glass bar"""
        chart = '&bar_glass%s=%s,%s,%s,%s,%s,%s&\r\n' % (
            self.line_index(), alpha, offset, colour, colour_outline, text, size)
        self.lines.append(chart)

    def pie(self, alpha, colour, style=''):
        """Create a pie chart"""
        self.pie_data = '%s,%s,%s' % (alpha, colour, style)
        
    def scatter(self, data, line_width, colour, text='', size=''):
        """Add scatter"""
        chart = '&scatter%s=%s,%s,%s,%s&\r\n' % (
            self.line_index(), line_width, colour, text, size)
        self.lines.append(chart)
        self.set_data(data)

    def hlc(self, data, alpha, line_width, colour, text='', size=''):
        """Add hlc"""
        chart = '&hlc%s=%s,%s,%s,%s,%s&\r\n' % (
            self.line_index(), alpha, line_width, colour, text, size)
        self.lines.append(chart)
        self.set_data(data)

    def candle(self, data, alpha, line_width, colour, text='', size=''):
        """Add hlc"""
        chart = '&candle%s=%s,%s,%s,%s,%s&\r\n' % (
            self.line_index(), alpha, line_width, colour, text, size)
        self.lines.append(chart)
        self.set_data(data)

    def render(self):
        """Render the graph string"""
        tmp = ''
		
        if self.title_text:
            tmp += '&title=%s,%s&\r\n' % (self.title_text, self.title_style)
		
        if self.x_legend:
            tmp += '&x_legend=%s,%s,%s\r\n' % (self.x_legend, self.x_legend_size, self.x_legend_colour)
		
        if self.x_label_style_size > 0 :
            tmp += '&x_label_style=%s,%s,%s,%s&\r\n' % (self.x_label_style_size,
                                                        self.x_label_style_colour,
                                                        self.x_label_style_orientation,
                                                        self.x_label_style_step)
		
        if self.x_tick_size > 0:
            tmp += "&x_ticks=%s&\r\n" % self.x_tick_size

        if self.x_axis_steps > 0:
            tmp += "&x_axis_steps=%s&\r\n" % self.x_axis_steps

        if self.x_axis_3d > 0:
            tmp += "&x_axis_3d=%s&\r\n" % self.x_axis_3d

        if self.y_legend:
            tmp += '&y_legend=%s,%s,%s&\r\n' % (self.y_legend, self.y_legend_size, self.y_legend_colour)

        if self.y_right_legend:
            tmp += '&y2_legend=%s,%s,%s&\r\n' % (self.y_right_legend, self.y_right_legend_size,
            self.y_right_legend_colour)

        if self.y_label_style_size == 'none':
            tmp += "&y_label_style=%s&\r\n" % self.y_label_style_size
        elif self.y_label_style_size > 0:
            tmp += "&y_label_style=%s" % self.y_label_style_size
            if self.y_label_style_colour:
                tmp += ",%s&\r\n" % (self.y_label_style_colour)

        tmp += '&y_ticks=5,10,%s&\r\n' % self.y_steps
		
        if not self.lines:
            tmp += self.line_default
        else:
            tmp += ''.join(self.lines)
        tmp += ''.join(self.data)
		
        if self.y2_lines:
            tmp += '&y2_lines=%s&\r\n' % ",".join([str(value) for value in self.y2_lines])
            tmp += '&show_y2=true&\r\n'
		
        if self.x_labels:
            tmp += '&x_labels=%s&\r\n' % ",".join([str(label) for label in self.x_labels])
        else:
            if self.x_min:
                tmp += '&x_min=%s&\r\n' % self.x_min
            if self.x_max:
                tmp += '&x_max=%s&\r\n' % self.x_max		
		
        tmp += '&y_min=%s&\r\n' % self.y_min
        tmp += '&y_max=%s&\r\n' % self.y_max
        if self.y2_min > 0:
            tmp += '&y2_min=%s&\r\n' % self.y2_min
			
        if self.y2_max > 0:
            tmp += '&y2_max=%s&\r\n' % self.y2_max

        if self.bg_colour:
            tmp += '&bg_colour=%s&\r\n' % self.bg_colour

        if self.bg_image:
            tmp += '&bg_image=%s&\r\n' % self.bg_image
            tmp += '&bg_image_x=%s&\r\n' % self.bg_image_x
            tmp += '&bg_image_y=%s&\r\n' % self.bg_image_y

        if self.x_axis_colour:
            tmp += '&x_axis_colour=%s&\r\n' % self.x_axis_colour
            tmp += '&x_grid_colour=%s&\r\n' % self.x_grid_colour

        if self.y_axis_colour:
            tmp += '&y_axis_colour=%s&\r\n' % self.y_axis_colour
            tmp += '&y_grid_colour=%s&\r\n' % self.y_grid_colour

        if self.y_right_axis_colour:
            tmp += '&y2_axis_colour=%s&\r\n' % self.y_right_axis_colour

        if self.x_offset:
            tmp += '&x_offset=%s&\r\n' % self.x_offset

        if self.inner_bg_colour:
            tmp += '&inner_background=%s' % self.inner_bg_colour
            if self.inner_bg_colour_2:
                tmp += ',%s,%s' % (self.inner_bg_colour_2, self.inner_bg_angle)
            tmp += "&\r\n"

        if self.pie_data:
            tmp += '&pie=%s&\r\n' % self.pie_data
            tmp += self.pie_value
            tmp += self.pie_labels
            tmp += self.pie_colours
            tmp += self.pie_links

        if self.tool_tip:
            tmp += '&tool_tip=%s&\r\n' % self.tool_tip
            
        if self.y_format:
            tmp += '&y_format=%s&\r\n' % self.y_format
			
        if self.num_decimals:
            tmp += '&num_decimals=%s&\r\n' % self.num_decimals
			
        if self.is_fixed_num_decimals_forced:
            tmp += '&is_fixed_num_decimals_forced=%s&\r\n' % self.is_fixed_num_decimals_forced
			
        if self.is_decimal_separator_comma:
            tmp += '&is_decimal_separator_comma=%s&\r\n' % self.is_decimal_separator_comma
			
        if self.is_thousand_separator_disabled:
            tmp += '&is_thousand_separator_disabled=%s&\r\n' % self.is_thousand_separator_disabled
			
        return tmp
