#!/usr/bin/python
"""Class test for openflashchart"""

import doctest
import unittest

from openflashchart import Graph
from openflashchart import GraphObject

class GraphTest(unittest.TestCase):

    def test_render(self):
        g = Graph()
        self.assertEquals(g.render(), '&x_axis_steps=1&\r\n&y_ticks=5,10,5&\r\n&line=3,#87421F&\r\n&y_min=0&\r\n&y_max=20&\r\n')

    def test_line_index(self):
        g = Graph()
        self.assertEquals(g.line_index(), '')
        g.lines.append('')
        self.assertEquals(g.line_index(), '_2')

    def test_set_data(self):
        g = Graph()
        g.set_data([1, 3, 4])
        self.assertEquals(g.data[0], '&values=1,3,4&\r\n')
        g.set_data(['bla', 'bla'])
        self.assertEquals(g.data[1], '&values_2=bla,bla&\r\n')

class GraphObjectTest(unittest.TestCase):

    def test_render(self):
        res_1 = ('<object classid="clsid:d27cdb6e-ae6d-11cf-96b8-444553540000" '
                 'codebase="http://fpdownload.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=8,0,0,0" '
                 'width="200" height="200" id="ie_graph" align="middle">'
                 '<param name="allowScriptAccess" value="sameDomain" />'
                 '<param name="movie" value="/open-flash-chart.swf?width=200&height=200&data=" />'
                 '<param name="quality" value="high" />'
                 '<param name="bgcolor" value="#FFFFFF" />'
                 '<embed src="/open-flash-chart.swf?width=200&height=200&data=" quality="high" '
                 'bgcolor="#FFFFFF" width="200" height="200" id="graph" align="middle" '
                 'allowScriptAccess="sameDomain" type="application/x-shockwave-flash" '
                 'pluginspage="http://www.macromedia.com/go/getflashplayer" /></object>')
        res_2 = ('<object classid="clsid:d27cdb6e-ae6d-11cf-96b8-444553540000" '
                 'codebase="http://fpdownload.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=8,0,0,0" '
                 'width="100" height="100" id="ie_custom_graph" align="middle">'
                 '<param name="allowScriptAccess" value="sameDomain" />'
                 '<param name="movie" value="/swf/open-flash-chart.swf?width=100&height=100&data=stats.txt" />'
                 '<param name="quality" value="high" />'
                 '<param name="bgcolor" value="#FFFFFF" />'
                 '<embed src="/swf/open-flash-chart.swf?width=100&height=100&data=stats.txt" quality="high" '
                 'bgcolor="#FFFFFF" width="100" height="100" id="custom_graph" align="middle" '
                 'allowScriptAccess="sameDomain" type="application/x-shockwave-flash" '
                 'pluginspage="http://www.macromedia.com/go/getflashplayer" /></object>')

        go = GraphObject.render()
        self.assertEquals(go, res_1)

        go = GraphObject.render(100, 100, 'stats.txt', '/swf/', 'custom_graph')
        self.assertEquals(go, res_2)


flags = (doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE |
         doctest.REPORT_ONLY_FIRST_FAILURE)

def test_suite():
    tests = [unittest.makeSuite(GraphTest),
             unittest.makeSuite(GraphObjectTest),
             doctest.DocFileTest('doc.txt', optionflags=flags)]
    return unittest.TestSuite(tests)

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')
